"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.emit = exports.inspect = void 0;
var http_1 = __importDefault(require("http"));
var https_1 = __importDefault(require("https"));
var stream_1 = require("stream");
var zlib_1 = __importDefault(require("zlib"));
var type_is_1 = __importDefault(require("type-is"));
var destroy_1 = __importDefault(require("destroy"));
var contant_1 = require("./contant");
var utils_1 = require("./utils");
var currentListener = null;
function inspect(listener) {
    currentListener = listener;
}
exports.inspect = inspect;
function emit(event) {
    if (currentListener) {
        currentListener(event);
    }
}
exports.emit = emit;
function inspectedMethod(func) {
    var rest = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        rest[_i - 1] = arguments[_i];
    }
    if (!currentListener) {
        return func.apply(this, rest);
    }
    var req = func.apply(this, rest);
    inspectRequest(req);
    return req;
}
;
function inspectRequest(request) {
    Promise.all([
        collectRemoteAddress(request),
        collectReq(request),
        collectRes(request),
    ]).then(function (_a) {
        var remoteAddress = _a[0], req = _a[1], res = _a[2];
        if (remoteAddress) {
            req.remoteAddress = remoteAddress;
            res.cost = Date.now() - req.time;
        }
        if (req.type && req.rawBody) {
            req.textBody = req.rawBody.toString('utf8');
        }
        emit({ type: 'http/https', req: req, res: res });
    }).catch(function (error) {
        console.log(error);
    });
}
function collectRemoteAddress(request) {
    return new Promise(function (resolve) {
        request.prependOnceListener('socket', function (socket) {
            socket.on('connect', function () { var _a; return resolve((_a = socket.remoteAddress) !== null && _a !== void 0 ? _a : null); });
            socket.on('error', function () { return resolve(null); });
            socket.on('close', function () { return resolve(null); });
        });
    });
}
function collectReq(request) {
    return new Promise(function (resolve, reject) {
        var _a;
        var headers = request.getHeaders();
        var req = {
            time: Date.now(),
            headers: headers,
            type: (0, utils_1.hasbody)(headers)
                ? type_is_1.default.is(String((_a = headers['content-type']) !== null && _a !== void 0 ? _a : ''), contant_1.CONTENT_TYPES) || undefined
                : undefined,
            method: request.method,
            remoteAddress: '',
            url: "".concat(request.protocol, "//").concat(request.getHeader('host')).concat(request.path),
        };
        if (req.type) {
            var requestBody_1 = [];
            var reqWrite_1 = request.write;
            request.write = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                /**
                 * chunk can be either a string or a Buffer.
                 */
                var chunk = args[0];
                if (Buffer.isBuffer(chunk)) {
                    requestBody_1.push(chunk);
                }
                else {
                    requestBody_1.push(Buffer.from(chunk, 'utf8'));
                }
                return reqWrite_1.apply(this, args);
            };
            var reqEnd_1 = request.end;
            request.end = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                /**
                 * the first argument might be a callback or a chunk
                 */
                var maybeChunk = args[0];
                if (Buffer.isBuffer(maybeChunk)) {
                    requestBody_1.push(maybeChunk);
                }
                else if (maybeChunk && typeof maybeChunk !== 'function') {
                    requestBody_1.push(Buffer.from(maybeChunk, 'utf8'));
                }
                return reqEnd_1.apply(this, args);
            };
            request.once('finish', function () {
                if (!requestBody_1.length) {
                    reject(null);
                }
                else {
                    req.rawBody = Buffer.concat(requestBody_1);
                    resolve(req);
                }
            });
            request.once('error', function () { return reject(null); });
        }
        else {
            resolve(req);
        }
    });
}
function collectRes(request) {
    return new Promise(function (resolve, reject) {
        request.prependOnceListener('response', function (response) {
            var _a, _b, _c, _d;
            var res = {
                status: (_a = response.statusCode) !== null && _a !== void 0 ? _a : 0,
                headers: response.headers,
                type: (0, utils_1.hasbody)(response.headers)
                    ? type_is_1.default.is((_b = response.headers['content-type']) !== null && _b !== void 0 ? _b : '', contant_1.CONTENT_TYPES) || undefined
                    : undefined,
                cost: 0,
            };
            if (res.type) {
                var input_1 = new stream_1.PassThrough();
                var decoders_1 = [];
                var encodings = String((_c = res.headers['content-encoding']) !== null && _c !== void 0 ? _c : '')
                    .split(',')
                    .map(function (val) { return val.trim(); })
                    .filter(function (val) { return val.length; });
                for (var _i = 0, encodings_1 = encodings; _i < encodings_1.length; _i++) {
                    var encoding = encodings_1[_i];
                    switch (encoding) {
                        case 'gzip':
                        case 'x-gzip':
                        case 'compress':
                        case 'x-compress':
                        case 'deflate':
                            decoders_1.push(zlib_1.default.createUnzip(contant_1.zlibOptions));
                            break;
                        case 'br':
                            decoders_1.push(zlib_1.default.createBrotliDecompress(contant_1.brotliOptions));
                            break;
                    }
                }
                var timeout_1 = setTimeout(function () { return resolve(res); }, 1000);
                var push_1 = response.push;
                response.push = function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    if (timeout_1 !== null) {
                        clearTimeout(timeout_1);
                        timeout_1 = null;
                    }
                    input_1.push.apply(input_1, args);
                    return push_1.apply(this, args);
                };
                response.prependOnceListener('end', function () {
                    input_1.end();
                });
                for (var i = 0; i < decoders_1.length; i++) {
                    ((_d = decoders_1[i - 1]) !== null && _d !== void 0 ? _d : input_1).pipe(decoders_1[i]);
                }
                var output = decoders_1.length ? decoders_1[decoders_1.length - 1] : input_1;
                var responseBody_1 = [];
                output.on('data', function (chunk) {
                    responseBody_1.push(chunk);
                });
                output.on('end', function () {
                    res.rawBody = Buffer.concat(responseBody_1);
                    (0, destroy_1.default)(input_1);
                    for (var _i = 0, decoders_2 = decoders_1; _i < decoders_2.length; _i++) {
                        var decoder = decoders_2[_i];
                        (0, destroy_1.default)(decoder);
                    }
                    resolve(res);
                });
                output.on('error', function () {
                    (0, destroy_1.default)(input_1);
                    for (var _i = 0, decoders_3 = decoders_1; _i < decoders_3.length; _i++) {
                        var decoder = decoders_3[_i];
                        (0, destroy_1.default)(decoder);
                    }
                });
            }
            else {
                resolve(res);
            }
        });
        request.once('error', function (error) {
            reject({ error: error });
        });
    });
}
http_1.default.request = inspectedMethod.bind(http_1.default, http_1.default.request);
http_1.default.get = inspectedMethod.bind(http_1.default, http_1.default.get);
https_1.default.request = inspectedMethod.bind(https_1.default, https_1.default.request);
https_1.default.get = inspectedMethod.bind(https_1.default, https_1.default.get);
//# sourceMappingURL=index.js.map